/* global deg2rad */
function unrefract (pr, tr, alt) {
  var LTLIM = 14.5, GELIM = 15.5;

  var aadeg = alt / deg2rad;

  if (aadeg < LTLIM)
    return unrefractLT15 (pr, tr, alt);
  else if (aadeg >= GELIM)
    return unrefractGE15 (pr, tr, alt);
  else {
    /* smooth blend -- important for inverse */
    var taLT = unrefractLT15 (pr, tr, alt);
    var taGE = unrefractGE15 (pr, tr, alt);
    var p = (aadeg - LTLIM) / (GELIM - LTLIM);
    return taLT + (taGE - taLT) * p;
  }
}

function unrefractGE15 (pr, tr, alt) {
  var r = 7.888888e-5 * pr / ((273 + tr) * Math.tan(alt));
  return  alt - r;
}

function unrefractLT15 (pr, tr, alt) {
  var aadeg = alt / deg2rad;

  var a = ((2e-5 * aadeg + 1.96e-2) * aadeg + 1.594e-1) * pr;
  var b = (273 + tr) * ((8.45e-2 * aadeg + 5.05e-1) * aadeg + 1);
  var r = a/b * deg2rad;

  return (alt < 0 && r < 0) ? alt : alt - r;  /* 0 below ~5 degs */
}

/* correct the true altitude, ta, for refraction to the apparent altitude, aa,
 * each in radians, given the local atmospheric pressure, pr, in mbars, and
 * the temperature, tr, in degrees C.
 */
function refract (ta) {
  var MAXERR = 0.1 / 3600.0 * deg2rad;   /* desired accuracy, rads */

  var pr = 1013.25, tr = 10, 
      d, t, t0, a;

  /* first guess of error is to go backwards.
   * make use that we know delta-apparent is always < delta-true.
   */
  t = unrefract (pr, tr, ta);
  d = 0.8 * (ta - t);
  t0 = t;
  a = ta;

  /* use secant method to discover a value that unrefracts to ta.
   * max=7 ave=2.4 loops in hundreds of test cases.
   */
  while (1) {
    a += d;
    t = unrefract (pr, tr, a);
    if (Math.abs(ta - t) <= MAXERR) break;
    d *= -(ta - t) / (t0 - t);
    t0 = t;
  }

  return a;
}

